@@ -10,6 +10,8 @@ from kabaret.subprocess_manager.flow import RunAction
 
 from libreflow import baseflow
 
+from libreflow.utils.b3d import wrap_python_expr
+
 class Department(baseflow.departments.Department):
     _short_name = flow.Param(None)
     _file_prefix = flow.Computed(cached=True)
@@ -255,7 +257,7 @@ class BuildLayoutScene(RunAction):
         
         # Get scene builder arguments
         frame_start = 101
-        frame_end = 101 + self._shot_data.get("nb_frames", 0)
+        frame_end = 101 + self._shot_data.get("nb_frames", 0) - 1
         resolution_x = 2048
         resolution_y = 858
         fps = 24
@@ -284,7 +286,7 @@ class BuildLayoutScene(RunAction):
         return [
             "-b", template_path,
             "--addons", "io_import_images_as_planes,camera_plane,lfs_scene_builder,add_camera_rigs",
-            "--python-expr", python_expr
+            "--python-expr", wrap_python_expr(python_expr)
         ]
     
     def extra_env(self):
