@@ -6,6 +6,7 @@ from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
 from kabaret.subprocess_manager.flow import RunAction
 
 from libreflow import baseflow
+from .file import FileSystemMap
 
 
 class Department(baseflow.departments.Department):
@@ -48,6 +49,80 @@ class RiggingDepartment(Department):
     _short_name = flow.Param("rig")
 
 
+class Util(flow.Object):
+
+    _util_map = flow.Parent()
+    path = flow.Computed().ui(hidden=True)
+    _file_prefix = flow.Computed()
+    files = flow.Child(FileSystemMap)
+
+    def compute_child_value(self, child_value):
+        if child_value is self.path:
+            settings = get_contextual_dict(self, "settings")
+            path = os.path.join(
+                self._util_map.path.get(),
+                self.name(),
+            )
+            self.path.set(path)
+        elif child_value is self._file_prefix:
+            self._file_prefix.set(self._util_map._file_prefix.get() + self.name() + '_')
+
+
+class UtilMap(flow.Map):
+
+    _short_name = flow.Param(None)
+    _utils = flow.Parent()
+    _file_prefix = flow.Computed()
+    path = flow.Computed().ui(hidden=True)
+    create = flow.Child(baseflow.maputils.SimpleCreateAction)
+    
+    @classmethod
+    def mapped_type(cls):
+        return Util
+    
+    def compute_child_value(self, child_value):
+        if child_value is self.path:
+            settings = get_contextual_dict(self, "settings")
+            path = os.path.join(
+                self._utils.path.get(),
+                self._short_name.get(),
+            )
+            self.path.set(path)
+        elif child_value is self._file_prefix:
+            self._file_prefix.set(self._utils._file_prefix.get() + self._short_name.get() + '_')
+
+
+class SkinningUtils(UtilMap):
+    
+    _short_name = flow.Param("skinning")
+
+
+class RigModuleUtils(UtilMap):
+    
+    _short_name = flow.Param("rigmod")
+
+
+class Utils(flow.Object):
+    
+    skinning = flow.Child(SkinningUtils)
+    rig_modules = flow.Child(RigModuleUtils)
+    path = flow.Computed().ui(hidden=True)
+    _file_prefix = flow.Computed()
+
+    def compute_child_value(self, child_value):
+        if child_value is self.path:
+            settings = get_contextual_dict(self, "settings")
+            path = os.path.join(
+                "lib",
+                settings["asset_type"],
+                "utils"
+            )
+            self.path.set(path)
+        elif child_value is self._file_prefix:
+            settings = get_contextual_dict(self, "settings")
+            child_value.set("lib_{asset_type}_utils_".format(**settings))
+
+
 class AssetDepartments(flow.Object):
     design = flow.Child(DesignDepartment)
     modeling = flow.Child(ModelingDepartment)
@@ -92,12 +167,13 @@ class ConfigureAssetFileAction(flow.Action):
         label="Source path",
         placeholder="Drop here a valid file/folder"
     )
-    create_as = flow.Param("working copy", RevisionType).ui(
+    create_as = flow.Param("new publication", RevisionType).ui(
         label="Create as"
     )
 
     def get_buttons(self):
         self.message.set("")
+        self.path.revert_to_default()
         return ["Confirm", "Cancel"]
 
     def run(self, button):
@@ -376,6 +452,7 @@ class AssetFamilies(flow.Map):
 class AssetType(flow.Object):
 
     asset_families = flow.Child(AssetFamilies).ui(expanded=True)
+    utils = flow.Child(Utils).ui(expanded=False)
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
@@ -419,25 +496,39 @@ class CreateAssetAction(flow.Action):
     def asset_family_short_name(self, name):
         return name
     
-    def revert_to_defaults(self):
-        self.type.revert_to_default()
-        self.family.revert_to_default()
-
     def child_value_changed(self, child_value):
         if child_value is self.asset_name:
             self.type.touch()
             self.family.touch()
     
     def compute_child_value(self, child_value):
-        kitsu_api = self.root().project().kitsu_api()
-        asset_name = self.asset_name.get()
-        asset_data = kitsu_api.get_asset_data(asset_name)
-        asset_type = kitsu_api.get_asset_type(asset_name)
+        self.message.set("<h2>Create an asset</h2>")
 
-        if child_value is self.type:                                                                                           
-            self.type.set(self.asset_type_short_name(asset_type["name"]) if asset_type else "")
-        elif child_value is self.family:
-            self.family.set(self.asset_family_short_name(asset_data["data"]["family"]) if asset_data else "")
+        if child_value in [self.type, self.family]:
+            asset_name = self.asset_name.get()
+
+            if not asset_name:
+                self.message.set(self.message.get() + "<br>")
+                child_value.set("")
+                return
+            
+            kitsu_api = self.root().project().kitsu_api()
+            asset_data = kitsu_api.get_asset_data(asset_name)
+            asset_type = kitsu_api.get_asset_type(asset_name)
+
+            if not asset_data:
+                msg = f"No asset named {asset_name} in Kitsu."
+                msg = f"<font color=#D5000D>{msg}</font><br>"
+                self.message.set(self.message.get() + msg)
+                child_value.set("")
+                return
+            
+            self.message.set(self.message.get() + "<br>")
+
+            if child_value is self.type:
+                self.type.set(self.asset_type_short_name(asset_type["name"]) if asset_type else "")
+            elif child_value is self.family:
+                self.family.set(self.asset_family_short_name(asset_data["data"]["family"]) if asset_data else "")
     
     def _asset_oid(self):
         return self.root().project().oid() + "/asset_lib/asset_types/%s/asset_families/%s/assets/%s" % (
@@ -447,7 +538,8 @@ class CreateAssetAction(flow.Action):
         )
     
     def get_buttons(self):
-        self.message.set("<h2>Create an asset</h2>")
+        self.message.set("<h2>Create an asset</h2><br>")
+        self.asset_name.revert_to_default()
         return ["Create", "Cancel"]
     
     def run(self, button):
@@ -468,13 +560,13 @@ class CreateAssetAction(flow.Action):
         # Warn if asset isn't registered in Kitsu
         if not asset_type_name or not asset_family_name:
             if not asset_name:
-                msg = "Asset name must not be empty"
+                msg = "Asset name must not be empty."
             else:
                 msg = f"No asset named <b>{asset_name}</b> in Kitsu."
 
             self.message.set((
-                "<h3>Configure asset</h3>"                
-                f"<font color=#D5000D>{msg}</font>"
+                "<h2>Create an asset</h2>"                
+                f"<font color=#D5000D>{msg}</font><br>"
             ))
             return self.get_result(close=False)
 
